local http = require 'resty.http'
local cjson = require "cjson.safe"
local responses = require 'kong.tools.responses'

local _M = {}

local PREFIX = 'X-'

function _M.execute(config)
  local httpc = http:new()
  local headers = ngx.req.get_headers()
  local client_body_reader, _ = httpc:get_client_body_reader()
  for key, value in pairs(config.headers or {}) do
    headers[key] = value
  end
  local port = ngx.var.server_port
  headers['X-Target-Method'] = ngx.var.request_method
  headers['X-Target-Scheme'] = ngx.var.scheme
  headers['X-Target-Host'] = ngx.var.host
  headers['X-Target-Port'] = port 
  headers['X-Target-Path'] = ngx.var.uri
  headers['X-Target-Uri'] = ngx.var.request_uri
  headers['host'] = nil
  local res, err = httpc:request_uri(config.url, {
    method = config.method or ngx.var.request_method,
    ssl_verify = false,
    headers = headers,
    body = client_body_reader,
    query = ngx.req.get_uri_args(),
  })
  if err ~= nil then
    return responses.send(ngx.HTTP_INTERNAL_SERVER_ERROR, err)
  end
  if res.status > 299 then
    local decoded = cjson.decode(res.body)
    local body = decoded
    if not body then
      body = res.body
    end
    return responses.send(res.status, body, res.headers)
  end
  for key, value in pairs(res.headers) do
    if string.sub(key, 1, string.len(PREFIX)) == PREFIX then
      ngx.req.set_header(key, value)
    end
  end
end

return _M
